@@ -3,11 +3,12 @@
 
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
-    Copyright 2009-2010, Alan Calvert
+    Copyright 2009-2011, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of version 2 of the GNU General Public
-    License as published by the Free Software Foundation.
+    it and/or modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either version 2 of
+    the License, or (at your option) any later version.
 
     yoshimi is distributed in the hope that it will be useful, but WITHOUT ANY
     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -18,10 +19,9 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is a derivative of a ZynAddSubFX original, modified December 2010
+    This file is derivative of original ZynAddSubFX code, modified March 2011
 */
 
-#include <iostream>
 #include <cmath>
 
 #include "Misc/Config.h"
@@ -37,7 +37,7 @@ void Microtonal::defaults(void)
     octavesize = 12;
     Penabled = 0;
     PAnote = 69;
-    PAfreq = 440.0;
+    PAfreq = 440.0f;
     Pscaleshift = 64;
 
     Pfirstkey = 0;
@@ -61,80 +61,90 @@ void Microtonal::defaults(void)
     octave[11].x2 = 1;
     Pname = string("12tET");
     Pcomment = string("Equal Temperament 12 notes per octave");
-    Pglobalfinedetune = 64;
+    Pglobalfinedetune = 64.0;
 }
 
-// Get the size of the octave
-// Get the frequency according the note number
-float Microtonal::getnotefreq(int note, int keyshift)
+// Get the frequency according to the note number
+float Microtonal::getNoteFreq(int note, int keyshift)
 {
     // in this function will appears many times things like this:
     // var=(a+b*100)%b
     // I had written this way because if I use var=a%b gives unwanted results when a<0
     // This is the same with divisions.
 
-    if((Pinvertupdown != 0) && ((Pmappingenabled == 0) || (Penabled == 0)))
-        note = (int) Pinvertupdowncenter * 2 - note;
+    if ((Pinvertupdown != 0) && ((Pmappingenabled == 0) || (Penabled == 0)))
+        note = Pinvertupdowncenter * 2 - note;
 
-    //compute global fine detune
-    float globalfinedetunerap = powf(2.0f, (Pglobalfinedetune - 64.0f) / 1200.0f); //-64.0 .. 63.0 cents
+    // compute global fine detune, -64.0 .. 63.0 cents
+    float globalfinedetunerap =
+        (Pglobalfinedetune > 64.0f || Pglobalfinedetune < 64.0f)
+            ? pow(2.0f, (Pglobalfinedetune - 64.0f) / 1200.0f)
+            : 1.0f;
+    // was float globalfinedetunerap = powf(2.0f, (Pglobalfinedetune - 64.0f) / 1200.0f);
 
-    if(Penabled == 0)
-        return powf(2.0f, (note - PAnote + keyshift) / 12.0f) * PAfreq * globalfinedetunerap;                      //12tET
+    if (!Penabled)
+    {
+        return getNoteFreq(note + keyshift) * globalfinedetunerap;
+    }
 
     int scaleshift = (Pscaleshift - 64 + octavesize * 100) % octavesize;
 
-    //compute the keyshift
+    // compute the keyshift
     float rap_keyshift = 1.0f;
-    if(keyshift != 0) {
+    if (keyshift)
+    {
         int kskey = (keyshift + octavesize * 100) % octavesize;
         int ksoct = (keyshift + octavesize * 100) / octavesize - 100;
-        rap_keyshift  = (kskey == 0) ? 1.0f : (octave[kskey - 1].tuning);
+        rap_keyshift  = (!kskey) ? 1.0f : (octave[kskey - 1].tuning);
         rap_keyshift *= powf(octave[octavesize - 1].tuning, ksoct);
     }
 
-    //if the mapping is enabled
-    if(Pmappingenabled != 0) {
-        if((note < Pfirstkey) || (note > Plastkey))
-            return -1.0;
-        //Compute how many mapped keys are from middle note to reference note
-        //and find out the proportion between the freq. of middle note and "A" note
-        int tmp = PAnote - Pmiddlenote;
-        bool minus = false;
-        if(tmp < 0) {
+    // if the mapping is enabled
+    if (Pmappingenabled)
+    {
+        if ((note < Pfirstkey) || (note > Plastkey))
+            return -1.0f;
+        // Compute how many mapped keys are from middle note to reference note
+        // and find out the proportion between the freq. of middle note and "A" note
+        int tmp = PAnote - Pmiddlenote, minus = 0;
+        if (tmp < 0)
+        {
             tmp   = -tmp;
-            minus = true;
+            minus = 1;
         }
         int deltanote = 0;
-        for(int i = 0; i < tmp; i++)
+        for (int i = 0; i < tmp; ++i)
             if (Pmapping[i % Pmapsize] >= 0)
                 deltanote++;
-        float rap_anote_middlenote = (deltanote == 0) ? 1.0f : (octave[(deltanote - 1) % octavesize].tuning);
-        if (deltanote != 0)
-            rap_anote_middlenote *= powf(octave[octavesize - 1].tuning, (deltanote - 1) / octavesize);
+        float rap_anote_middlenote =
+            (deltanote == 0) ? (1.0f) : (octave[(deltanote - 1) % octavesize].tuning);
+        if(deltanote != 0)
+            rap_anote_middlenote *= powf(octave[octavesize - 1].tuning,
+                                         (deltanote - 1) / octavesize);
         if (minus)
             rap_anote_middlenote = 1.0f / rap_anote_middlenote;
 
-        //Convert from note (midi) to degree (note from the tunning)
-        int degoct = (note - Pmiddlenote + (int) Pmapsize * 200) / (int)Pmapsize - 200;
-        int degkey = (note - Pmiddlenote + (int)Pmapsize * 100) % Pmapsize;
+        // Convert from note (midi) to degree (note from the tunning)
+        int degoct = (note - Pmiddlenote + Pmapsize * 200)
+                      / Pmapsize - 200;
+        int degkey = (note - Pmiddlenote + Pmapsize * 100) % Pmapsize;
         degkey = Pmapping[degkey];
-        if(degkey < 0)
-            return -1.0f;           //this key is not mapped
+        if (degkey < 0) // this key is not mapped
+            return -1.0f;
 
-        //invert the keyboard upside-down if it is asked for
-        //TODO: do the right way by using Pinvertupdowncenter
-        if(Pinvertupdown != 0)
+        // invert the keyboard upside-down if it is asked for
+        // TODO: do the right way by using Pinvertupdowncenter
+        if (Pinvertupdown)
         {
             degkey = octavesize - degkey - 1;
             degoct = -degoct;
         }
-        //compute the frequency of the note
+        // compute the frequency of the note
         degkey  = degkey + scaleshift;
         degoct += degkey / octavesize;
         degkey %= octavesize;
 
-        float freq = (degkey == 0) ? 1.0f : octave[degkey - 1].tuning;
+        float freq = (degkey == 0) ? (1.0f) : octave[degkey - 1].tuning;
         freq *= powf(octave[octavesize - 1].tuning, degoct);
         freq *= PAfreq / rap_anote_middlenote;
         freq *= globalfinedetunerap;
@@ -142,16 +152,18 @@ float Microtonal::getnotefreq(int note, int keyshift)
             freq /= octave[scaleshift - 1].tuning;
         return freq * rap_keyshift;
     }
-    else  //if the mapping is disabled
-    {   int nt = note - PAnote + scaleshift;
-        int ntkey     = (nt + octavesize * 100) % octavesize;
-        int ntoct     = (nt - ntkey) / octavesize;
+    else // if the mapping is disabled
+    {
+        int nt = note - PAnote + scaleshift;
+        int ntkey = (nt + octavesize * 100) % octavesize;
+        int ntoct = (nt - ntkey) / octavesize;
 
         float oct  = octave[octavesize - 1].tuning;
-        float freq = octave[(ntkey + octavesize - 1) % octavesize].tuning *powf(oct, ntoct) * PAfreq;
-        if(ntkey == 0)
+        float freq = octave[(ntkey + octavesize - 1) % octavesize].tuning
+                     * powf(oct, ntoct) * PAfreq;
+        if (ntkey == 0)
             freq /= oct;
-        if(scaleshift != 0)
+        if (scaleshift != 0)
             freq /= octave[scaleshift - 1].tuning;
 //	fprintf(stderr,"note=%d freq=%.3f cents=%d\n",note,freq,(int)floor(log(freq/PAfreq)/log(2.0)*1200.0+0.5));
         freq *= globalfinedetunerap;
@@ -164,7 +176,7 @@ float Microtonal::getnotefreq(int note, int keyshift)
 int Microtonal::linetotunings(unsigned int nline, const char *line)
 {
     int x1 = -1, x2 = -1, type = -1;
-    float x = -1.0, tmp, tuning = 1.0;
+    float x = -1.0f, tmp, tuning = 1.0f;
     if (strstr(line, "/") == NULL)
     {
         if (strstr(line, ".") == NULL)
@@ -176,7 +188,7 @@ int Microtonal::linetotunings(unsigned int nline, const char *line)
         else
         {   // float number case
             sscanf(line, "%f", &x);
-            if (x < 0.000001)
+            if (x < 0.000001f)
                 return 1;
             type = 1; // float type(cents)
         }
@@ -204,9 +216,9 @@ int Microtonal::linetotunings(unsigned int nline, const char *line)
     {
         case 1:
             x1 = (int) floorf(x);
-            tmp = fmodf(x, (float)1.0);
-            x2 = (int)(floorf(tmp * 1e6));
-            tuning = powf(2.0, x / 1200.0);
+            tmp = fmodf(x, 1.0f);
+            x2 = lrintf(floorf(tmp * 1e6f));
+            tuning = powf(2.0f, x / 1200.0f);
             break;
         case 2:
             x = ((float)x1) / x2;
@@ -225,7 +237,8 @@ int Microtonal::linetotunings(unsigned int nline, const char *line)
 // Convert the text to tunnings
 int Microtonal::texttotunings(const char *text)
 {
-    unsigned int i, k = 0, nl = 0;
+    int i;
+    unsigned int k = 0, nl = 0;
     char *lin;
     lin = new char[MAX_LINE_SIZE + 1];
     while (k < strlen(text))
@@ -429,7 +442,7 @@ int Microtonal::loadkbm(string filename)
     // loads the reference freq.
     if (loadline(file, &tmp[0]))
         return 2;
-    float tmpPAfreq = 440.0;
+    float tmpPAfreq = 440.0f;
     if (!sscanf(&tmp[0], "%f", &tmpPAfreq))
         return 2;
     PAfreq = tmpPAfreq;
@@ -472,7 +485,7 @@ void Microtonal::add2XML(XMLwrapper *xml)
     xml->addparbool("invert_up_down_center", Pinvertupdowncenter);
 
     xml->addparbool("enabled", Penabled);
-    xml->addpar("global_fine_detune", Pglobalfinedetune);
+    xml->addpar("global_fine_detune", lrint(Pglobalfinedetune));
 
     xml->addpar("a_note", PAnote);
     xml->addparreal("a_freq", PAfreq);
@@ -526,7 +539,7 @@ void Microtonal::getfromXML(XMLwrapper *xml)
     Pinvertupdowncenter=xml->getparbool("invert_up_down_center", Pinvertupdowncenter);
 
     Penabled=xml->getparbool("enabled", Penabled);
-    Pglobalfinedetune=xml->getpar127("global_fine_detune", Pglobalfinedetune);
+    Pglobalfinedetune = xml->getpar127("global_fine_detune", Pglobalfinedetune);
 
     PAnote = xml->getpar127("a_note", PAnote);
     PAfreq = xml->getparreal("a_freq", PAfreq, 1.0, 10000.0);
