@@ -1,7 +1,7 @@
 /*
     AlsaJackClient.h - Alsa audio + Jack midi
-    
-    Copyright 2009-2010, Alan Calvert
+
+    Copyright 2009-2011, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -34,16 +34,12 @@ class AlsaJackClient : public MusicClient
         AlsaJackClient() : MusicClient() { };
         ~AlsaJackClient() { Close(); };
 
-        bool openAudio(WavRecord *recorder);
-        bool openMidi(WavRecord *recorder);
+        bool openAudio(void);
+        bool openMidi(void);
         bool Start(void);
-        void Close(void);
-        bool jacksessionReply(string cmdline) { return jackEngine.jacksessionReply(cmdline); }
+        void Close(void) { alsaEngine.Close(); jackEngine.Close(); }
         unsigned int getSamplerate(void) { return alsaEngine.getSamplerate(); };
         int getBuffersize(void) { return alsaEngine.getBuffersize(); };
-        int grossLatency(void)
-            { return alsaEngine.grossLatency() + jackEngine.grossLatency(); };
-
         string audioClientName(void) { return alsaEngine.audioClientName(); };
         string midiClientName(void) { return jackEngine.clientName(); };
         int audioClientId(void) { return alsaEngine.audioClientId(); };
