@@ -1,7 +1,7 @@
 /*
     JackEngine.cpp
 
-    Copyright 2009-2010, Alan Calvert
+    Copyright 2009-2011, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -17,8 +17,6 @@
     along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <iostream>
-
 #include <errno.h>
 #include <jack/midiport.h>
 #include <jack/thread.h>
@@ -28,15 +26,9 @@
 using namespace std;
 
 #include "Misc/Config.h"
-#include "Misc/SynthEngine.h"
 #include "MusicIO/JackEngine.h"
 
-JackEngine::JackEngine() :
-    MusicIO(),
-    #if defined(JACK_SESSION)
-        lastevent(NULL),
-    #endif
-    jackClient(NULL)
+JackEngine::JackEngine() : jackClient(NULL)
 {
     audio.jackSamplerate = 0;
     audio.jackNframes = 0;
@@ -48,8 +40,6 @@ JackEngine::JackEngine() :
     midi.port = NULL;
     midi.ringBuf = NULL;
     midi.pThread = 0;
-    midi.semName.clear();
-    midi.eventsUp = NULL;
 }
 
 
@@ -63,7 +53,7 @@ bool JackEngine::connectServer(string server)
             usleep(3333);
         }
     }
-    if (jackClient != NULL)
+    if (jackClient)
     {
         Runtime.setRtprio(jack_client_max_real_time_priority(jackClient));
         audio.jackSamplerate = jack_get_sample_rate(jackClient);
@@ -78,31 +68,43 @@ bool JackEngine::connectServer(string server)
 
 bool JackEngine::openJackClient(string server)
 {
+    int jopts = JackNullOption;
+    jack_status_t jstatus;
     string clientname = "yoshimi";
-    if (!Runtime.nameTag.empty())
+    if (Runtime.nameTag.size())
         clientname += ("-" + Runtime.nameTag);
-    bool named_server = !server.empty() && server.compare("default");
-    int jopts = ((named_server) ? JackServerName : JackNullOption)              // JackNoStartServer = 0x01
-                 | ((Runtime.startJack) ? JackNullOption : JackNoStartServer);  // JackServerName = 0x04
-    jack_status_t jstatus;                                                      // JackSessionID = 0x20
-    if (Runtime.doRestoreJackSession && !Runtime.jackSessionUuid.empty())
-    {
-        #if defined(JACK_SESSION)
+    bool named_server = server.size() > 0 && server.compare("default") != 0;
+    if (named_server)
+        jopts |= JackServerName;
+    if (!Runtime.startJack)
+        jopts |= JackNoStartServer;
+    #if defined(JACK_SESSION)
+        if (Runtime.restoreJackSession && Runtime.jackSessionUuid.size())
+        {
             jopts |= JackSessionID;
-        #endif
+            if (named_server)
+                jackClient = jack_client_open(clientname.c_str(), (jack_options_t)jopts,
+                                              &jstatus, Runtime.jackServer.c_str(),
+                                              Runtime.jackSessionUuid.c_str());
+            else
+                jackClient = jack_client_open(clientname.c_str(), (jack_options_t)jopts,
+                                              &jstatus, Runtime.jackSessionUuid.c_str());
+        }
+        else
+        {
+            if (named_server)
+                jackClient = jack_client_open(clientname.c_str(), (jack_options_t)jopts,
+                                              &jstatus, Runtime.jackServer.c_str());
+            else
+                jackClient = jack_client_open(clientname.c_str(), (jack_options_t)jopts, &jstatus);
+        }
+    #else
         if (named_server)
             jackClient = jack_client_open(clientname.c_str(), (jack_options_t)jopts,
-                                          &jstatus, Runtime.jackServer.c_str(),
-                                          Runtime.jackSessionUuid.c_str());
+                                          &jstatus, Runtime.jackServer.c_str());
         else
-            jackClient = jack_client_open(clientname.c_str(), (jack_options_t)jopts,
-                                          &jstatus, Runtime.jackSessionUuid.c_str());
-    }
-    else if (named_server)
-        jackClient = jack_client_open(clientname.c_str(), jack_options_t(jopts),
-                                      NULL, Runtime.jackServer.c_str());
-    else
-        jackClient = jack_client_open(clientname.c_str(), jack_options_t(jopts), NULL);
+            jackClient = jack_client_open(clientname.c_str(), (jack_options_t)jopts, &jstatus);
+    #endif
     if (jackClient)
         return true;
     else
@@ -115,9 +117,9 @@ bool JackEngine::Start(void)
 {
     jack_set_error_function(_errorCallback);
     jack_set_xrun_callback(jackClient, _xrunCallback, this);
-
     #if defined(JACK_SESSION)
-        if (jack_set_session_callback && jack_set_session_callback(jackClient, _jsessionCallback, this))
+        if (jack_set_session_callback
+            && jack_set_session_callback(jackClient, _jsessionCallback, this))
             Runtime.Log("Set jack session callback failed");
     #endif
 
@@ -127,32 +129,23 @@ bool JackEngine::Start(void)
         goto bail_out;
     }
 
-    synth->actionLock(lockmute);
-    if (NULL != midi.port)
+    if (midi.port && !Runtime.startThread(&midi.pThread, _midiThread, this, true, true))
     {
-        int chk = 999;
-        pthread_attr_t attr;
-        if (setThreadAttributes(&attr, true, true))
-        {
-            if ((chk = pthread_create(&midi.pThread, &attr, _midiThread, this)))
-                Runtime.Log("Failed to start jack midi thread (sched_fifo): " + asString(chk));
-        }
-        if (chk)
-        {
-            if (!setThreadAttributes(&attr, false))
-                goto bail_out;
-            if ((chk = pthread_create(&midi.pThread, &attr, _midiThread, this)))
-            {
-                Runtime.Log("Failed to start jack midi thread (sched_other): " + asString(chk));
-                goto bail_out;
-            }
-        }
+            Runtime.Log("Failed to start jack midi thread");
+            goto bail_out;
     }
+
+    if (!latencyPrep())
+    {
+        Runtime.Log("Jack latency prep failed ");
+        goto bail_out;
+    }
+
     if (!jack_activate(jackClient)
         && NULL != audio.ports[0]
         && NULL != audio.ports[1])
     {
-        if (Runtime.connectJackaudio && !connectJackPorts())
+        if (!Runtime.restoreJackSession && Runtime.connectJackaudio && !connectJackPorts())
         {
             Runtime.Log("Failed to connect jack audio ports");
             goto bail_out;
@@ -163,11 +156,9 @@ bool JackEngine::Start(void)
         Runtime.Log("Failed to activate jack client");
         goto bail_out;
     }
-    synth->actionLock(unlock);
     return true;
 
 bail_out:
-    synth->actionLock(unlock);
     Close();
     return false;
 }
@@ -175,9 +166,6 @@ bail_out:
 
 void JackEngine::Close(void)
 {
-    if (NULL != midi.port && midi.pThread)
-        if (pthread_cancel(midi.pThread))
-            Runtime.Log("Failed to cancel Jack midi thread");
     if (NULL != jackClient)
     {
         int chk;
@@ -202,32 +190,19 @@ void JackEngine::Close(void)
             midi.ringBuf = NULL;
         }
         jackClient = NULL;
-        MusicIO::Close();
     }
 }
 
 
-bool JackEngine::openAudio(WavRecord *recorder)
+bool JackEngine::openAudio(void)
 {
     const char *portnames[] = { "left", "right" };
     for (int port = 0; port < 2; ++port)
-    {
         audio.ports[port] = jack_port_register(jackClient, portnames[port],
-                                              JACK_DEFAULT_AUDIO_TYPE,
-                                              JackPortIsOutput, 0);
-    }
-    if (NULL != audio.ports[0] && NULL != audio.ports[1])
-    {
-        if (prepBuffers(false))
-        {
-            wavRecorder = recorder;
-            jack_port_set_latency (audio.ports[0], jack_get_buffer_size(jackClient));
-            jack_port_set_latency (audio.ports[1], jack_get_buffer_size(jackClient));
-            jack_recompute_total_latencies (jackClient);
-            audioLatency = jack_port_get_latency(audio.ports[0]);
-            return true;
-        }
-    }
+                                               JACK_DEFAULT_AUDIO_TYPE,
+                                               JackPortIsOutput, 0);
+    if (audio.ports[0] && audio.ports[1])
+        return prepBuffers(false) && latencyPrep();
     else
         Runtime.Log("Failed to register jack audio ports");
     Close();
@@ -235,45 +210,24 @@ bool JackEngine::openAudio(WavRecord *recorder)
 }
 
 
-bool JackEngine::openMidi(WavRecord *recorder)
+bool JackEngine::openMidi(void)
 {
-    if (NULL != jackClient)
+    const char *port_name = "midi in";
+    midi.port = jack_port_register(jackClient, port_name,
+                                   JACK_DEFAULT_MIDI_TYPE,
+                                   JackPortIsInput, 0);
+    if (!midi.port)
     {
-        midi.ringBuf =
-            jack_ringbuffer_create(sizeof(struct midi_event) * 4096);
-        if (NULL != midi.ringBuf)
-        {
-            midi.semName = string(clientName());
-            midi.eventsUp = sem_open(midi.semName.c_str(), O_CREAT, S_IRWXU, 0);
-            if (midi.eventsUp != SEM_FAILED)
-            {
-                const char *port_name = "midi in";
-                midi.port = jack_port_register(jackClient, port_name,
-                                               JACK_DEFAULT_MIDI_TYPE,
-                                               JackPortIsInput, 0);
-                if (NULL != midi.port)
-                {
-                    wavRecorder = recorder;
-                    jack_port_set_latency (midi.port, jack_get_buffer_size(jackClient));
-                    jack_recompute_total_latencies (jackClient);
-                    midiLatency = jack_port_get_latency (midi.port);
-                    return true;
-                }
-                else
-                    Runtime.Log("Failed to register jack midi port");
-            }
-            else
-                Runtime.Log("Failed to create jack midi semaphore "
-                            + midi.semName + string(strerror(errno)));
-        }
-        else
-            Runtime.Log("Failed to create jack midi ringbuffer");
-
+        Runtime.Log("Failed to register jack midi port");
+        return false;
     }
-    else
-        Runtime.Log("NULL jackClient through registerMidi");
-    Close();
-    return false;
+    midi.ringBuf = jack_ringbuffer_create(sizeof(struct midi_event) * 4096);
+    if (!midi.ringBuf)
+    {
+        Runtime.Log("Failed to create jack midi ringbuffer");
+        return false;
+    }
+    return true;
 }
 
 
@@ -281,7 +235,7 @@ bool JackEngine::connectJackPorts(void)
 {
     const char** playback_ports = jack_get_ports(jackClient, NULL, NULL,
                                                  JackPortIsPhysical|JackPortIsInput);
-	if (playback_ports == NULL)
+	if (!playback_ports)
     {
         Runtime.Log("No physical jack playback ports found.");
         return false;
@@ -304,7 +258,7 @@ bool JackEngine::connectJackPorts(void)
 
 int JackEngine::clientId(void)
 {
-    if (NULL != jackClient)
+    if (jackClient)
         return jack_client_thread_id(jackClient);
     else
         return -1;
@@ -313,7 +267,7 @@ int JackEngine::clientId(void)
 
 string JackEngine::clientName(void)
 {
-    if (NULL != jackClient)
+    if (jackClient)
         return string(jack_get_client_name(jackClient));
     else
         Runtime.Log("clientName() with null jackClient");
@@ -332,9 +286,9 @@ int JackEngine::processCallback(jack_nframes_t nframes)
     bool okaudio = true;
     bool okmidi = true;
 
-    if (NULL != midi.port)
+    if (midi.port)
         okmidi = processMidi(nframes);
-    if (NULL != audio.ports[0] && NULL != audio.ports[1])
+    if (audio.ports[0] && audio.ports[1])
         okaudio = processAudio(nframes);
     return (okaudio && okmidi) ? 0 : -1;
 }
@@ -346,7 +300,7 @@ bool JackEngine::processAudio(jack_nframes_t nframes)
     {
         audio.portBuffs[port] =
             (float*)jack_port_get_buffer(audio.ports[port], nframes);
-        if (NULL == audio.portBuffs[port])
+        if (!audio.portBuffs[port])
         {
             Runtime.Log("Failed to get jack audio port buffer: " + asString(port));
             return false;
@@ -354,12 +308,9 @@ bool JackEngine::processAudio(jack_nframes_t nframes)
     }
     memset(audio.portBuffs[0], 0, sizeof(float) * nframes);
     memset(audio.portBuffs[1], 0, sizeof(float) * nframes);
-    if (NULL != synth)
-    {
-        getAudio();
-        memcpy(audio.portBuffs[0], zynLeft, sizeof(float) * nframes);
-        memcpy(audio.portBuffs[1], zynRight, sizeof(float) * nframes);
-    }
+    getAudio();
+    memcpy(audio.portBuffs[0], zynLeft, sizeof(float) * nframes);
+    memcpy(audio.portBuffs[1], zynRight, sizeof(float) * nframes);
     return true;
 }
 
@@ -367,62 +318,52 @@ bool JackEngine::processAudio(jack_nframes_t nframes)
 bool JackEngine::processMidi(jack_nframes_t nframes)
 {
     void *portBuf = jack_port_get_buffer(midi.port, nframes);
-    if (NULL == portBuf)
+    if (!portBuf)
     {
-        Runtime.Log("Bad get jack midi port buffer");
+        Runtime.Log("Bad midi jack_port_get_buffer");
         return  false;
     }
 
-    unsigned int byt;
     unsigned int idx;
     unsigned int act_write;
     struct midi_event event;
     char *data;
     unsigned int wrote = 0;
     unsigned int tries = 0;
-    int chk;
     jack_midi_event_t jEvent;
     jack_nframes_t eventCount = jack_midi_get_event_count(portBuf);
     for(idx = 0; idx < eventCount; ++idx)
     {
         if(!jack_midi_event_get(&jEvent, portBuf, idx))
         {
-            // no interest in 0 size or long events
-            if (jEvent.size > 0 && jEvent.size <= sizeof(event.data))
+            if (jEvent.size < 1 || jEvent.size > sizeof(event.data))
+                continue; // no interest in zero sized or long events
+            event.time = jEvent.time;
+            memset(event.data, 0, sizeof(event.data));
+            memcpy(event.data, jEvent.buffer, jEvent.size);
+            wrote = 0;
+            tries = 0;
+            data = (char*)&event;
+            while (wrote < sizeof(struct midi_event) && tries < 3)
             {
-                event.time = jEvent.time;
-                for (byt = 0; byt < jEvent.size; ++byt)
-                    event.data[byt] = jEvent.buffer[byt];
-                while (byt < sizeof(event.data))
-                    event.data[byt++] = 0;
-                wrote = 0;
-                tries = 0;
-                data = (char*)&event;
-                while (wrote < sizeof(struct midi_event) && tries < 3)
-                {
-                    act_write =
-                        jack_ringbuffer_write(midi.ringBuf, (const char*)data,
-                                              sizeof(struct midi_event) - wrote);
-                    wrote += act_write;
-                    data += act_write;
-                    ++tries;
-                }
-                if (wrote == sizeof(struct midi_event))
-                {
-                    if (Runtime.runSynth && midi.eventsUp)
-                    {
-                        if ((chk = sem_post(midi.eventsUp)))
-                        Runtime.Log("Jack midi semaphore post failed: "
-                                    + string(strerror(errno)));
-                    }
-                }
-                else
-                {
-                    Runtime.Log("Bad write to midi ringbuffer: "
-                                + asString(wrote) + " / "
-                                + asString((int)sizeof(struct midi_event)));
-                    return false;
-                }
+                act_write = jack_ringbuffer_write(midi.ringBuf, (const char*)data,
+                                                  sizeof(struct midi_event) - wrote);
+                wrote += act_write;
+                data += act_write;
+                ++tries;
+            }
+            if (wrote == sizeof(struct midi_event))
+            {
+                if (sem_post(&midiSem) < 0)
+                    Runtime.Log("processMidi semaphore post error, "
+                                + string(strerror(errno)));
+            }
+            else
+            {
+                Runtime.Log("Bad write to midi ringbuffer: "
+                            + asString(wrote) + " / "
+                            + asString((int)sizeof(struct midi_event)));
+                return false;
             }
         }
         else
@@ -443,7 +384,7 @@ int JackEngine::_xrunCallback(void *arg)
 
 void JackEngine::_errorCallback(const char *msg)
 {
-    Runtime.Log("Jack error report:" + string(msg));
+    Runtime.Log("Jack reports error: " + string(msg));
 }
 
 
@@ -455,32 +396,34 @@ void *JackEngine::_midiThread(void *arg)
 void *JackEngine::midiThread(void)
 {
     unsigned char channel, note, velocity;
-    int chk;
     int ctrltype;
     int par = 0;
     unsigned int fetch;
     unsigned int ev;
     struct midi_event midiEvent;
-    pthread_cleanup_push(_midiCleanup, this);
+    if (sem_init(&midiSem, 0, 0) < 0)
+    {
+        Runtime.Log("Error on jack midi sem_init " + string(strerror(errno)));
+        return false;
+    }
+
     while (Runtime.runSynth)
     {
-        pthread_testcancel();
-        if ((chk = sem_wait(midi.eventsUp)))
+        if (sem_wait(&midiSem) < 0)
         {
-            Runtime.Log("Error on jack midi semaphore wait: "
+            Runtime.Log("midiThread semaphore wait error, "
                         + string(strerror(errno)));
             continue;
         }
-        pthread_testcancel();
-        fetch = jack_ringbuffer_read(midi.ringBuf, (char*)&midiEvent,
-                                     sizeof(struct midi_event));
+        if (!Runtime.runSynth)
+            break;
+        fetch = jack_ringbuffer_read(midi.ringBuf, (char*)&midiEvent, sizeof(struct midi_event));
         if (fetch != sizeof(struct midi_event))
         {
             Runtime.Log("Short ringbuffer read, " + asString(fetch) + " / "
                         + asString((int)sizeof(struct midi_event)));
             continue;
         }
-        pthread_testcancel();
         channel = midiEvent.data[0] & 0x0F;
         switch ((ev = midiEvent.data[0] & 0xF0))
         {
@@ -502,26 +445,6 @@ void *JackEngine::midiThread(void)
                 setMidiController(channel, ctrltype, par);
                 break;
 
-            case 0x0C: // program change ... but how?
-                Runtime.Log("How to change to program " + asString(par)
-                            + " on channel " + asString(channel) + "?");
-                par = midiEvent.data[2];
-                break;
-
-            case 0x42: // Sostenuto On/Off
-                {
-                    par = midiEvent.data[2]; // < 63 off, > 64 on
-                    string errstr = string("Sostenuto ");
-                    if (par < 63)
-                        errstr += "off";
-                    else if (par > 64)
-                        errstr += "on";
-                    else
-                        errstr += "?";
-                    Runtime.Log(errstr);
-                }
-                break;
-
             case 0x78: // all sound off
                 ctrltype = C_allsoundsoff;
                 setMidiController(channel, ctrltype, 0);
@@ -539,7 +462,6 @@ void *JackEngine::midiThread(void)
 
             case 0x80: // note-off
                 note = midiEvent.data[1];
-                // velocity = midiEvent.data[2];
                 setMidiNote(channel, note);
                 break;
 
@@ -571,49 +493,84 @@ void *JackEngine::midiThread(void)
                 break;
         }
     }
-    pthread_cleanup_pop(1);
+    sem_destroy(&midiSem);
     return NULL;
 }
 
 
-void JackEngine::_midiCleanup(void *arg)
+bool JackEngine::latencyPrep(void)
 {
-    static_cast<JackEngine*>(arg)->midiCleanup();
+#if defined(JACK_LATENCY)  // >= 0.120.1 API
+
+    if (jack_set_latency_callback(jackClient, _latencyCallback, this))
+    {
+        Runtime.Log("Set latency callback failed");
+        return false;
+    }
+    return true;
+
+#else // < 0.120.1 API
+
+    if (jack_port_set_latency && audio.ports[0] && audio.ports[1])
+    {
+        jack_port_set_latency(audio.ports[0], jack_get_buffer_size(jackClient));
+        jack_port_set_latency(audio.ports[1], jack_get_buffer_size(jackClient));
+        if (jack_recompute_total_latencies)
+            jack_recompute_total_latencies(jackClient);
+    }
+    return true;
+
+#endif
 }
 
-
-void JackEngine::midiCleanup(void)
-{
-    int chk;
-    if (NULL != midi.eventsUp && (chk = sem_close(midi.eventsUp)))
-        Runtime.Log("Failed to close jack midi semaphore "
-                    + midi.semName + string(strerror(errno)));
-    midi.eventsUp = NULL;
-    if (!midi.semName.empty() && (chk = sem_unlink(midi.semName.c_str())))
-        Runtime.Log("Failed to unlink jack midi semaphore "
-                    + midi.semName + string(strerror(errno)));
-}
-
-
 #if defined(JACK_SESSION)
-    void JackEngine::_jsessionCallback(jack_session_event_t *event, void *arg)
+
+void JackEngine::_jsessionCallback(jack_session_event_t *event, void *arg)
+{
+    return static_cast<JackEngine*>(arg)->jsessionCallback(event);
+}
+
+void JackEngine::jsessionCallback(jack_session_event_t *event)
+{
+    string uuid = string(event->client_uuid);
+    string filename = string("yoshimi-") + uuid + string(".xml");
+    string filepath = string(event->session_dir) + filename;
+    Runtime.setJackSessionSave((int)event->type, filepath);
+    string cmd = Runtime.programCmd() + string(" -U ") + uuid
+                 + string(" -u ${SESSION_DIR}") + filename;
+    event->command_line = strdup(cmd.c_str());
+    if (jack_session_reply(jackClient, event))
+        Runtime.Log("Jack session reply failed");
+    jack_session_event_free(event);
+}
+
+#endif
+
+
+#if defined(JACK_LATENCY)
+
+void JackEngine::_latencyCallback(jack_latency_callback_mode_t mode, void *arg)
+{
+    return static_cast<JackEngine*>(arg)->latencyCallback(mode);
+}
+
+
+void JackEngine::latencyCallback(jack_latency_callback_mode_t mode)
+{
+    if (mode == JackCaptureLatency)
     {
-        return static_cast<JackEngine*>(arg)->jsessionCallback(event);
+        if (audio.ports[0] && audio.ports[1])
+        {
+            jack_latency_range_t range[2];
+            for (int i = 0; i < 2; ++i)
+            {
+                jack_port_get_latency_range(audio.ports[i], mode, &range[i]);
+                range[i].min++;
+                range[i].max += audio.jackNframes;
+                jack_port_set_latency_range(audio.ports[i], JackPlaybackLatency, &range[i]);
+            }
+        }
     }
-    
-    
-    void JackEngine::jsessionCallback(jack_session_event_t *event)
-    {
-        lastevent = event;
-        Runtime.setJackSessionSave(event->type, event->session_dir, event->client_uuid);
-    }
-    
-    
-    bool JackEngine::jacksessionReply(string cmdline)
-    {
-        lastevent->command_line = strdup(cmdline.c_str());
-        bool ok = !jack_session_reply(jackClient, lastevent);
-        jack_session_event_free (lastevent);
-        return ok;
-    }
+}
+
 #endif
