@@ -6,8 +6,9 @@
     Copyright 2009, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of version 2 of the GNU General Public
-    License as published by the Free Software Foundation.
+    it and/or modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either version 2 of
+    the License, or (at your option) any later version.
 
     yoshimi is distributed in the hope that it will be useful, but WITHOUT ANY
     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -25,7 +26,7 @@
 
 SUBnoteParameters::SUBnoteParameters() : Presets()
 {
-    setpresettype("Psubsyth");
+    setpresettype("SUBnoteParameters");
     AmpEnvelope = new EnvelopeParams(64, 1);
     AmpEnvelope->ADSRinit_dB(0, 40, 127, 25);
     FreqEnvelope = new EnvelopeParams(64, 0);
@@ -39,12 +40,12 @@ SUBnoteParameters::SUBnoteParameters() : Presets()
     defaults();
 }
 
+
 void SUBnoteParameters::defaults(void)
 {
     PVolume = 96;
-    PPanning = 64;
+    setPan(PPanning = 64);
     PAmpVelocityScaleFunction = 90;
-
     Pfixedfreq = 0;
     PfixedfreqET = 0;
     Pnumstages = 2;
@@ -78,15 +79,31 @@ void SUBnoteParameters::defaults(void)
     GlobalFilterEnvelope->defaults();
 }
 
+
 SUBnoteParameters::~SUBnoteParameters()
 {
-    delete (AmpEnvelope);
-    delete (FreqEnvelope);
-    delete (BandWidthEnvelope);
-    delete (GlobalFilter);
-    delete (GlobalFilterEnvelope);
+    delete AmpEnvelope;
+    delete FreqEnvelope;
+    delete BandWidthEnvelope;
+    delete GlobalFilter;
+    delete GlobalFilterEnvelope;
 }
 
+
+void SUBnoteParameters::setPan(char pan)
+{
+    PPanning = pan;
+    if (!randomPan())
+    {
+        float t = (float)(PPanning - 1) / 126.0f;
+        pangainL = cosf(t * PI / 2.0f);
+        pangainR = cosf((1.0f - t) * PI / 2.0f);
+    }
+    else
+        pangainL = pangainR = 0.7f;
+}
+
+
 void SUBnoteParameters::add2XML(XMLwrapper *xml)
 {
     xml->addpar("num_stages",Pnumstages);
@@ -156,6 +173,7 @@ void SUBnoteParameters::add2XML(XMLwrapper *xml)
     xml->endbranch();
 }
 
+
 void SUBnoteParameters::getfromXML(XMLwrapper *xml)
 {
     Pnumstages=xml->getpar127("num_stages",Pnumstages);
