@@ -3,11 +3,12 @@
 
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
-    Copyright 2009, Alan Calvert
+    Copyright 2009-2011, Alan Calvert
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of version 2 of the GNU General Public
-    License as published by the Free Software Foundation.
+    it and/or modify it under the terms of the GNU Library General Public
+    License as published by the Free Software Foundation; either version 2 of
+    the License, or (at your option) any later version.
 
     yoshimi is distributed in the hope that it will be useful, but WITHOUT ANY
     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -18,7 +19,7 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is a derivative of a ZynAddSubFX original, modified October 2009
+    This file is derivative of ZynAddSubFX original code, modified January 2011
 */
 
 #ifndef LFO_H
@@ -39,11 +40,11 @@ class LFO
         float incx, incrnd, nextincrnd;
         float amp1, amp2; // used for randomness
         float lfointensity;
-        float lfornd, lfofreqrnd;
+        float lfornd;
+        float lfofreqrnd;
         float lfodelay;
         char lfotype;
         int freqrndenabled;
 };
 
 #endif
-
